@@ -16,8 +16,10 @@ unsigned int hdup_log = 0;
 unsigned int ignore_tar_err = 0;
 unsigned int ignore_conf_err = 0;
 unsigned int dryrun = 0;
+unsigned int patched_tar = 0;
 sig_atomic_t hdup_pipe = 0;
 sig_atomic_t hdup_sig  = 0;
+sig_atomic_t hdup_alrm = 0;
 /* order of scheme's is important */
 char *scheme[] = {"daily","weekly","monthly","remote","restore"};
 char *subvar[] = {"%h","%a","%s","%e","%u","%c","%g"};
@@ -44,7 +46,7 @@ backup_local(phost_t host[], int which, struct argument_t *arg)
 		hdup_setup(host, which);
 		/* hdup date dir is not needed when moving it to a
 		 * remote location */
-		rmdir ( host[which]->dirname_date );
+		rmdir (host[which]->dirname_date);
 	}
 
 	/* this also works with encryption */
@@ -66,7 +68,7 @@ backup_local(phost_t host[], int which, struct argument_t *arg)
 	/* set a lock, also when we're piping the tar to ssh  */
 	/* 15-09-2003: do the locking here, before any copying, MG 
 	 * at least before hdup_inclist() */
-	(void) hdup_lock(host[which], LOCK);
+	(void)hdup_lock(host[which], LOCK);
 
 	/* sort out the incremental files, hdup_inclist will return the scheme
 	 * that is actually used. If there is no previous backup list, hdup
@@ -114,10 +116,10 @@ backup_local(phost_t host[], int which, struct argument_t *arg)
 }
 
 int 
-main (int argc, char *argv[]) 
+main(int argc, char *argv[]) 
 {
 	phost_t host[MAXHOST + 1];
-	struct argument_t arg; int c = 0; unsigned int which; 
+	struct argument_t arg; int c = 0; int which;
 	char *cp;
 	char *logstring;
 	struct group *grp;
@@ -125,7 +127,7 @@ main (int argc, char *argv[])
 	struct tab algs[] = ALGORITHMS;
 
 #ifdef HAVE_GETOPT_H
-	static struct option long_options[] = 
+	static struct option long_options[] =
 	{ 
 		{"config", required_argument, 0, 'c'},
 		{"specific", required_argument, 0, 's'},
@@ -135,6 +137,7 @@ main (int argc, char *argv[])
 		{"version", no_argument, 0, 'v'},
 		{"ignore-tar", no_argument, 0, 'i'},
 		{"ignore-conf", no_argument, 0, 'I'},
+		{"patched-tar", no_argument, 0, 'P'},
 		{"dryrun", no_argument, 0, 'd'},
 		{"debug", no_argument, 0, 'D'},
 		{0,0,0,0}
@@ -144,8 +147,8 @@ main (int argc, char *argv[])
 	progname = g_strdup(argv[0]);
 
 	/* reset options */
-	arg.configpath  = NULL; arg.scheme      = -1;   
-	arg.host        = NULL; arg.date        = NULL; 
+	arg.configpath  = NULL; arg.scheme      = SCHEME_ILL;
+	arg.host        = NULL; arg.date        = NULL;
 	arg.extractdir  = NULL; arg.extractfile = NULL;
 
 	/* catch SIGPIPE */
@@ -158,16 +161,36 @@ main (int argc, char *argv[])
 	sa.sa_handler = hdup_sigint;
 	sigaction(SIGINT, &sa, (struct sigaction *)NULL);
 
+	/* setup SIGALRM. Used transfering with ssh */
+	sa.sa_handler = hdup_sigalrm;
+	sigaction(SIGALRM, &sa, (struct sigaction *)NULL);
 
+	/* check if we are suid of sgid, if so QUIT! hdup isn't
+         * designed to do this kind of stuff
+	 */
+        if (((getuid() != geteuid()) || (getgid() != getegid()))) {
+		FATAL("%s %s", progname, "is not designed to be ran suid/sgid");
+	}
+
+	/* This is stupid to have - but somewhere I do something
+	 * stupid - this at least prevents core dumps.... :-( */
+	/* check argument lengths */
+	for(c = 0; c < argc; c++) {
+		if (strlen(argv[c]) > MAXPATHLEN) {
+			FATAL("%s", "Command line argument too long");
+		}
+	}
+	
 	/* in host[0] I put the global options, these can be overriden in the config file */
 #ifdef HAVE_GETOPT_H
-	while ( (c = getopt_long(argc, argv, "c:s:hqvViIdD", long_options, 0)) != -1) {
+	while ((c = getopt_long(argc, argv, "c:s:hqvViIdDP", long_options, 0)) != -1) {
 #else
-	while ( (c = getopt(argc, argv, "c:s:hqvViIdD")) != -1) {
+	while ((c = getopt(argc, argv, "c:s:hqvViIdDP")) != -1) {
 #endif /*HAVE_GETOPT_H*/
+
 		switch (c) { 
 			case 'c':
-				arg.configpath = optarg;
+				arg.configpath = strdup(optarg);
 				hdup_ok_char(arg.configpath);
 				break;
 			case 'h':
@@ -177,8 +200,12 @@ main (int argc, char *argv[])
 			case 'v':
 				version();
 				hdup_cleanup(0,NULL);
+				break;
 			case 'q':
 				quiet += 1;
+				/* quiet option */
+				if (quiet > 3)
+					FATAL("%s","You can specify up to three -q's");
 				break;
 			case 'V':
 				verbose--;
@@ -186,10 +213,10 @@ main (int argc, char *argv[])
 					verbose = 0;
 				break;
 			case 's':
-				arg.extractfile = optarg;
+				arg.extractfile = strdup(optarg);
 				/* if the first char eq '/' we chop it off
 				 * as tar does the same */
-				if ( arg.extractfile[0] == '/' ) 
+				if (arg.extractfile[0] == '/') 
 					arg.extractfile++;
 				hdup_ok_char(arg.extractfile);
 				break;
@@ -199,6 +226,9 @@ main (int argc, char *argv[])
 			case 'I':
 				ignore_conf_err = 1;
 				break;
+			case 'P':
+				patched_tar = 1;
+				break;
 			case 'd':
 				dryrun = 1;
 				break;
@@ -221,6 +251,7 @@ main (int argc, char *argv[])
 		hdup_cleanup(-1,NULL);
 	}
 
+	/* MIEK; not sure I want to keep this for 2.0.11 */
 	/* sanitize the command line */
 	for (c = 0; c < argc; c++ ) 
 		hdup_ok_char(argv[c]);
@@ -230,15 +261,22 @@ main (int argc, char *argv[])
 	if (arg.scheme == REMOTE)
 		progname = g_strconcat(progname, " (remote) ", NULL);
 
-	/* quiet option */
-	if ( quiet > 3 ) 
-		FATAL("%s","You can specify up to three -q's");
+	if (arg.scheme != REMOTE && arg.scheme != RESTORE) {
+		/* useless when receiving a bunch of files */
+		if (patched_tar == 0) {
+			LOG("%s", "Running without -P and probably a non patched tar");
+			LOG("%s", "This will lead to incomplete backups");
+		} else {
+			LOG("%s", "Running with -P, all is well");
+		}
+	}
 
 	/* first the config file to set things up,  -c option */
 	if ( arg.configpath == NULL ) {
 		VERBOSE("%s %s", "No -c switch using default configfile:", ETCFILE);
 		arg.configpath = ETCFILE;
 	}
+	
 	if (g_file_test(arg.configpath, G_FILE_TEST_IS_REGULAR)) {
 		/* access test purely here for better error output */
 		if (access(arg.configpath, R_OK))
@@ -249,14 +287,14 @@ main (int argc, char *argv[])
 
 	}  else 
 		/* there is no config file */
-		FATAL("%s","Config file could not be opened");
+		FATAL("%s %s","Config file could not be opened:", arg.configpath);
 
 	/* config checks */
-	if ( findhost("global",host) == -1 ) 
+	if (findhost("global", host) == -1) 
 		FATAL("%s", "No [global] section in configuration file");
 
-	/* still -1? */
-	if (arg.scheme == -1)
+	/* still ILL? */
+	if (arg.scheme == SCHEME_ILL)
 		FATAL("%s", "Scheme must be: daily, weekly, monthly, remote or restore");
 
 	if (arg.scheme != RESTORE && arg.extractfile != NULL) 
@@ -289,7 +327,7 @@ main (int argc, char *argv[])
 		arg.host = argv[1];
 
 	/* RESTORE */ 
-	if ( arg.scheme == RESTORE ) {
+	if (arg.scheme == RESTORE) {
 		/* restore a backup */
 		if (argv[1] != NULL) 
 			arg.host = argv[1];
@@ -327,7 +365,7 @@ main (int argc, char *argv[])
 
 	/* check host in config file */
 	which = findhost(arg.host, host);
-	if ( which == -1 ) 
+	if (which == -1) 
 		FATAL("%s %s: %s","Host:", arg.host, "Not defined in configuration file");
 
 	/* we are now ready to start the backup. Log this, makes the gen. email 
@@ -553,6 +591,9 @@ main (int argc, char *argv[])
 			host[which]->exclude[c] = host[0]->exclude[c];
 		}
 	}
+	/* always put the archive dir in the exclude list 
+	 * this is what skip used to do */
+	host[which]->exclude[c] = host[which]->archive;
 
 	/* check algorithm and key path, both must be set, or none */
 	if ( ( host[which]->alg == NULL && host[which]->keypath != NULL ) || 
@@ -682,7 +723,7 @@ out:
 	/* run the prerun script - do this way early. People like to mount their
 	 * backup filesystem before doing anything */
 	if ( host[which]->prerun != NULL ) {
-		if ( hdup_runpre(host[which], scheme, arg.scheme, NULL) != 0 ) {
+		if ( hdup_runpre(host[which], scheme, arg.scheme) != 0 ) {
 			SYSLOG_HW("%s","Prescript executed with errors");
 			FATAL("%s: %s",host[which]->name, "Prescript executed with errors");
 		}
@@ -704,7 +745,7 @@ out:
 
 		backup_local(host, which, &arg);
 
-	} else if ( arg.scheme == REMOTE ) {
+	} else if (arg.scheme == REMOTE) {
 		/* remote always reads from stdin  */
 		/* fix hostname argument for REMOTE */
 		host[which]->name = arg.host; /* FIXME: still necessary? */
@@ -720,9 +761,9 @@ out:
 		}
 
 		/* prerun scripts are run! 11-09-2003 */
-		backup_remote(host, which, &arg);
+		backup_remote(host, which);
 
-	} else if ( arg.scheme == RESTORE ) {
+	} else if (arg.scheme == RESTORE) {
 		char *arch;
 		arch = g_strdup_printf("%s/%s",host[which]->archive,arg.host);
 		if ( hdup_globfilecheck(arch, NULL) == 0 ) 
@@ -743,24 +784,21 @@ out:
 
 	/* init the log string at this point */
 	logstring = NULL;
-	if ( arg.scheme < RESTORE && host[which]->remote == NULL ) {
-		if ( host[which]->chunksize == NULL ) {
-			hdup_overview("Archive size", (char*) (hdup_size (host[which]->archivename)));
-			logstring = g_strdup_printf("%s", hdup_size (host[which]->archivename));
-		} else {
-			hdup_overview("Chunk size", host[which]->chunksize);
-			logstring = g_strdup_printf("%s %s", host[which]->chunksize, "chunks");
-		}
-	} else {
-		if ( arg.scheme == RESTORE )
-			logstring = "restore";
-		if ( host[which]->remote != NULL ) 
-			logstring = "remote";
+	if (arg.scheme != RESTORE) {
+		hdup_overview("Bytes written", (char*)hdup_humansize(host[which]->bytes));
 	}
+	if (host[which]->chunksize != NULL) {
+		logstring = g_strdup_printf("%s (chunked)", (char*)hdup_humansize(host[which]->bytes));
+	} else {
+		logstring = g_strdup_printf("%s", (char*)hdup_humansize(host[which]->bytes));
+	}
+
 
 	/* first unlock THEN perform the postrun script, always lock, remote,
 	 * restore or normal operations */
-	(void)hdup_lock(host[which], UNLOCK);
+	if (arg.scheme != RESTORE) {
+		(void)hdup_lock(host[which], UNLOCK);
+	}
 
 	if (hdup_sig) 
 		hdup_cleanup(-1, NULL);
@@ -769,7 +807,7 @@ out:
 	/* some people use the LOCK file on a mounted medium which gets
 	 * unmounted by a postrun script, so we must unlock before the
 	 * postrun script runs. */
-	if ( host[which]->postrun != NULL ) {
+	if (host[which]->postrun != NULL) {
 		/* we must do some postrun stuff */
 		if ( hdup_runpost(host[which], scheme, arg.scheme) != 0 ) {
 			/* something went wrong, non fatal error */
@@ -781,11 +819,12 @@ out:
 	/* time keeping */
 	host[which]->elapsed = time(NULL) -  host[which]->elapsed;
 	hdup_overview("Elapsed", hdup_time(host[which]->elapsed));
-	SYSLOG("SUCCESS, %s, %s, %s", host[which]->name, logstring,  hdup_time(host[which]->elapsed));
+	SYSLOG("SUCCESS, %s (%s): %s, %s", host[which]->name, scheme[arg.scheme],
+			logstring,  hdup_time(host[which]->elapsed));
 	if (arg.scheme < REMOTE ) 
 		hdup_overview("Status", "successfully performed backup");
 
-	if ( arg.scheme != REMOTE )
+	if (arg.scheme != REMOTE)
 		hdup_cleanup(0,NULL);
 	else 
 		/* no overview, normal exit */
