@@ -21,9 +21,9 @@
  *
  */
 
+#include"config.h"
 #include"hdup.h"
 #include"prototype.h"
-#include"config.h"
 
 extern unsigned int ignore_conf_err;
 
@@ -40,7 +40,7 @@ char *keyword[] =  {
 
 /* This doesn't do quoting, escaping, etc. I don't need that actually */
 int 
-readline (char *line, FILE *from, int lim, int *linecnt)
+readline(char *line, FILE *from, int lim, int *linecnt)
 {
 	char *l = line;
 	int c;
@@ -74,7 +74,7 @@ readline (char *line, FILE *from, int lim, int *linecnt)
 			*l = '\0'; return 0;
 		}
 		if (lim < 0) {
-			WARN("Line %d: %s", *linecnt, "Maximum line length exceeded.");
+			WARN("Config: line %d: %s", *linecnt, "Maximum line length exceeded.");
 			return 1;
 		}
 
@@ -118,16 +118,18 @@ config(char *path, phost_t host[])
 
 			if ( *line == '[' ) {
 				/* new host found */
-				if ( ++i > MAXHOST ) 
-					FATAL("Line %d: %s", linecnt, "Too many hosts defined in the configuration file");
+				if ( ++i > MAXHOST ) {
+					FATAL("Config: line %d: %s", linecnt, "Too many hosts defined in the configuration file");
+				}
 
 				host[i] = (phost_t) g_malloc (sizeof(host_t));
 
 				/* Solaris complains */
 				h = (char*)rindex(line,']');
 
-				if (h == NULL)
-					FATAL("Line %d: %s", linecnt, "Could not find closing \']\'");
+				if (h == NULL) {
+					FATAL("Config: line %d: %s", linecnt, "Could not find closing \']\'");
+				}
 				
 				/* not the first '[' */
 				line++;
@@ -141,7 +143,7 @@ config(char *path, phost_t host[])
 				/* the first entry MUST be [global], hdup depends on this */
 				if ( i == 0 && (!g_str_equal(host[0]->name, "global"))) {
 					/* SYSLOG is not opened defined here (yet) */
-					FATAL("Line %d: %s", linecnt, "First host in config file must be named [global]");
+					FATAL("Config: line %d: %s", linecnt, "First host in config file must be named [global]");
 				}
 
 				/* fill her up */
@@ -167,6 +169,7 @@ config(char *path, phost_t host[])
 				host[i]->free           = NULL;
 				host[i]->complevel      = NULL;
 				host[i]->nobackup	= NULL;
+				host[i]->tar_conf_opt	= NULL;
 
 				/* protocol stuff */
 				host[i]->proto_opt      = NULL;
@@ -206,11 +209,11 @@ config(char *path, phost_t host[])
 				/* char* Solaris fix */
 
 				if (i == -1)
-					FATAL("Line %d: %s", linecnt, "No [host] statement seen");
+					FATAL("Config: line %d: %s", linecnt, "No [host] statement seen");
 				
 				h = (char*)index(line,'=');
 				if ( h == NULL ) {
-					VERBOSE("Line %d: %s", linecnt, "Crap seen in config file - skipping");
+					VERBOSE("Config: line %d: %s", linecnt, "Crap seen in config file - skipping");
 					continue;
 				} else {
 					/* actual line lenght is thus
@@ -240,7 +243,7 @@ config(char *path, phost_t host[])
 							if ((host[i]->archive)[slash - 1 ] != '/') {
 								host[i]->archive = g_realloc(host[i]->archive, slash + 2);
 								/* add a closing / if not already there */
-								VVERBOSE("%s","Adding closing \'/\' to archive dir");
+								VVERBOSE("Config: line %d: %s", linecnt, "Adding closing \'/\' to archive dir");
 								(host[i]->archive)[slash] = '/';
 								(host[i]->archive)[slash + 1] = '\0';
 							}
@@ -286,6 +289,7 @@ config(char *path, phost_t host[])
 							break;
 						case KEY_SKIP:
 							host[i]->skip = yesno(right, keyword[KEY_SKIP], linecnt);
+							LOG("Config: line %d: %s", linecnt, "Skip is always enabled");
 							break;
 						case KEY_FORCE:
 							host[i]->force = yesno(right, keyword[KEY_FORCE], linecnt);
@@ -295,7 +299,7 @@ config(char *path, phost_t host[])
 							break;
 						case KEY_SPARSE:
 							host[i]->sparse = yesno(right, keyword[KEY_SPARSE], linecnt);
-							LOG("Line %d: %s", linecnt, "Sparse is always enabled");
+							LOG("Config: line %d: %s", linecnt, "Sparse is always enabled");
 							break;
 						case KEY_ALWAYS:
 							host[i]->always = yesno(right, keyword[KEY_ALWAYS], linecnt);
@@ -304,7 +308,7 @@ config(char *path, phost_t host[])
 							setvar(right, &host[i]->tar, linecnt);
 							break;
 						case KEY_FIND:
-							LOG("Line %d: %s", linecnt, "Find is depreciated");
+							LOG("Config: line %d: %s", linecnt, "Find is depreciated");
 							break;
 						case KEY_ALLOW:
 							host[i]->allow_remote = yesno(right, keyword[KEY_ALLOW], linecnt);
@@ -340,14 +344,20 @@ config(char *path, phost_t host[])
 							break;
 						default:
 							/* not a keyword */
-							LOG("Line %d: %s %s",linecnt, left," : is not a keyword");
-							if (!ignore_conf_err)
+							LOG("Config: line %d: %s %s",linecnt, left," : is not a keyword");
+							if (!ignore_conf_err) {
+								g_free(left);
+								g_free(right);
 								return 1;
+							}
 							break;
 					}
 				}
 			}
 		} /* end while */
+		g_free(left);
+		g_free(right);
+		
 		host[i+1] = (phost_t) g_malloc (sizeof(host_t));
 		host[i+1]->name = NULL; /* possible error: i+1 */
 	} else {
@@ -383,13 +393,13 @@ inherit(char *hostnames, phost_t curr, phost_t host[], int linecnt)
 
 	i = setlist(hostnames, hostlist, linecnt);
 	if (i < 0) {
-		LOG("Line %d: Not able to parse inherit list [%s] so ignoring", linecnt, hostnames);
+		LOG("Config: line %d: Not able to parse inherit list [%s] so ignoring", linecnt, hostnames);
 		return -1;
 	}
 	for (i = 0; hostlist[i] != NULL && hostlist[i][0] != '\0' && i < MAXDIR; i++) {
 		int h = findhost(hostlist[i], host);	
 		if (h < 0)
-			FATAL("Line %d: Cannot find hostname to inherit from [%s]", linecnt, hostlist[i]);
+			FATAL("Config: line %d: Cannot find hostname to inherit from [%s]", linecnt, hostlist[i]);
 
 		/* inherit from host[h] -> curr */
 		addlist(host[h]->path, curr->path);
@@ -431,13 +441,17 @@ inherit(char *hostnames, phost_t curr, phost_t host[], int linecnt)
 int 
 setvar (char *dir, char **var, int linecnt) 
 {
+	if (!dir) {
+		return 0;
+	}
+	
 	/* set var to dir */
 	if (strlen(dir) > MAXPATHLEN)
 		return -1;
 
 	if (pure_whitespace(dir) != -1) {
 		*var = NULL;
-		WARN("Line %d: Resetting keywords like this, will not work", linecnt);
+		WARN("Config: line %d: Resetting keywords like this, will not work", linecnt);
 		return 0;
 	}
 
@@ -476,7 +490,7 @@ yesno (char *what, char *which, int linecnt)
 	if ( strncmp(what, "true", 4) == 0 ) { return 1; }
 	if ( strncmp(what, "on", 2) == 0 ) { return 1; }
 
-	LOG("Line %d: No \'on/yes\' or \'off/no\' specified, going with \'off/no\' for %s", linecnt, which);
+	LOG("Config: line %d: No \'on/yes\' or \'off/no\' specified, going with \'off/no\' for %s", linecnt, which);
 	return 0;	/* default is off/no */
 }
 
@@ -502,7 +516,7 @@ setlist (char *dirpath, char *hostpath[], int linecnt)
 
 	if (pure_whitespace(dirpath) != -1) {
 		hostpath[0] = NULL;
-		WARN("Line %d: Resetting keywords like this, will not work", linecnt);
+		WARN("Config: line %d: Resetting keywords like this, will not work", linecnt);
 		return 0;
 	}
 	
@@ -577,9 +591,14 @@ int
 pure_whitespace(char *str)
 {
 	char *s;
+	
+	if (!str) {
+		return 0;
+	}
 
 	for(s = str; *s; s++) {
-		if (isblank(*s) == 0) {
+		/* avoid isblank because it's a gnu-ism. bug 22 */
+		if (*s != '\t' || *s != ' ') {
 			return -1;
 		}
 	}
