@@ -39,8 +39,8 @@ hdup_getdate(char * d, int format, int history)
 	time_t currenttime;
 	struct tm *ltime;
 
-	if ( history == 1 ) {
-		g_strlcpy(d, STATIC, strlen(STATIC));
+	if (history == 1) {
+		g_strlcpy(d, STATIC, strlen(STATIC) + 1);
 		return FALSE;
 	}
 
@@ -288,10 +288,14 @@ hdup_lock(phost_t host, int l)
 	 * in host->basename/LOCK
 	 */
 	time_t locktime;
-	char *lockfile;
 	int lockfd;
+	char *lockfile = NULL;
 
 	lockfile = g_strconcat(host->basename, "/LOCK", NULL);
+	if (!lockfile) {
+		SYSLOG_H("%s", "Internal error");
+		FATAL("%s", "Internal error. Could not get LOCK");
+	}
 
 	if (dryrun) {
 		switch(l) {
@@ -307,11 +311,11 @@ hdup_lock(phost_t host, int l)
 
 	switch (l) {
 		case LOCK:
-			lockfd = open(lockfile, O_EXCL | O_CREAT);
+			lockfd = open(lockfile, O_EXCL | O_CREAT, S_IRUSR | S_IWUSR);
 			if (-1 == lockfd) {
 				if (g_file_test(lockfile , G_FILE_TEST_IS_REGULAR)) {
 					/* check the creation time */
-					locktime=hdup_file_time(lockfile);
+					locktime = hdup_file_time(lockfile);
 					if (locktime != (time_t) 0) {
 						SYSLOG_H("LOCK in %s FOUND (%s)", host->basename, strtok(ctime(&locktime),"\n") );
 						FATAL("%s: LOCK in %s found (%s)", host->name, host->basename, strtok(ctime(&locktime), "\n") );
@@ -409,6 +413,7 @@ hdup_cleanup(int exitcode, phost_t host)
 	/* no explicit status error */
 	if ( exitcode > 0 ) {
 		WARN("%s", "AN ERROR OCCURED");
+		hdup_overview(NULL, NULL);
 		hdup_overview("AN ERROR OCCURED!", NULL);
 		hdup_overview("NOTHING WAS DONE!", NULL);
 		hdup_overview(NULL, NULL);
@@ -482,48 +487,6 @@ hdup_print(void)
 	}
 }
 
-char *
-hdup_size(char *filepath) 
-{
-	/* give back size in file,
-	 * 5k, 5M, 5G 
-	 */
-	off_t size; /* size in bytes */
-	float s = 0;
-	char *filesize;
-	struct stat buf;
-
-	if (filepath == NULL) 
-		return "?";
-
-	if (stat(filepath, &buf) == 0) {
-		size = buf.st_size;
-		if ( size > GIGABYTE ) {
-			s = (float) size / (float) GIGABYTE;
-			filesize = g_strdup_printf("%.1f%c",s, 'G');
-			return filesize;
-		}
-		else if (size > MEGABYTE) {
-			s = (float) size / (float) MEGABYTE;
-			filesize = g_strdup_printf("%.1f%c",s, 'M');
-			return filesize;
-		}
-		else if (size > KILOBYTE) {
-			s = (float) size / (float) KILOBYTE;
-			filesize = g_strdup_printf("%.0f%c",s, 'k');
-			return filesize;
-		}
-		if (size == 0) {
-			return "0";
-		} else {
-			filesize = g_strdup_printf("%d%c",(int)size, 'B');
-			return filesize;
-		}
-	} else {
-		return "0";
-	}
-}
-
 /* return size as a multiple of k, M or G */
 char *
 hdup_humansize(long long size) 
@@ -659,7 +622,7 @@ hdup_mkdir(char *dirpath, mode_t mode, char *user, char *group)
 	d = g_strconcat(dirpath, "/", NULL);
 
 	/* we can go 1 step too far with t++ */
-	for ( t = d ; *t != '\0' && (int)(t-d) <= l; t++ ) {
+	for (t = d ; *t != '\0' && (unsigned int)(t-d) <= l; t++) {
 		if ( *t == '/' ) {
 			*t = '\0'; /* make mkdir think the string stops here */
 			/* with an absolute path,we have the d is empty on the
@@ -1052,9 +1015,9 @@ hdup_subst(char *into, char *what, char *id)
 
 	/* get *what in between the two parts */
 	newstring = g_strdup_printf("%.*s%s%.*s",
-			lsize, left,
+			(int)lsize, left,
 			what,
-			rsize, right);
+			(int)rsize, right);
 
 	/* cannot free these values. See BUG #4 */
 #ifdef DEBUG
@@ -1121,7 +1084,6 @@ hdup_getheader(pheader_t head, FILE *stream)
 	/* read the header information */
 
 	/* NAME\nVERSION\nHOST\nSCHEME\nDATE\COMPR\nTARFILE\EOF */
-	/* TODO: better checking for lim , see readword */
 	hdup_readword(head->prot_name, stream, strlen(PROT_NAME));
 	hdup_readword(head->prot_ver, stream, strlen(PROT_NAME));
 	hdup_readword(head->hostname, stream, MAXPATHLEN);
@@ -1137,7 +1099,6 @@ hdup_getheader(pheader_t head, FILE *stream)
 	hdup_ok_char(head->hostname); hdup_ok_char(head->scheme);
 	hdup_ok_char(head->date); hdup_ok_char(head->compression);
 	hdup_ok_char(head->encryption); hdup_ok_char(head->extractdir);
-
 	return 0;
 }
 
@@ -1232,8 +1193,17 @@ hdup_sigint(int signal)
 	return;
 }
 
+void
+hdup_sigalrm(int signal)
+{
+	/* alarm went off - this is used during ssh transfers */
+	WARN("%s","SIGALRM (timeout) received, cleaning up");
+	hdup_alrm = 1;
+	return;
+}
+
 int 
-hdup_scheme ( char *sch ) 
+hdup_scheme(char *sch) 
 {
 	/* return the scheme sequence number */
 	int s = -1;
@@ -1277,7 +1247,7 @@ hdup_readword(char *word, FILE *from, int lim)
 }
 
 int 
-hdup_ok_char (char * check) 
+hdup_ok_char(char *check)
 {
 	/* remove dangerous characters */
 	/* see: http://www.colorado.edu/htmltools/sanitize.html */
@@ -1292,7 +1262,7 @@ hdup_ok_char (char * check)
 }
 
 long long int 
-hdup_free (char *path) 
+hdup_free(char *path) 
 {
 	/* check to see how many bytes are free on the backup file
 	 * system */
@@ -1396,27 +1366,29 @@ usage()
 	fprintf(stdout,"    @USER@REMOTE_HOST");
 	fprintf(stdout,"    restore to this host using USER\n");
 
-#ifdef LONG
+#ifdef HAVE_GETOPT_H
 	fprintf(stdout,"\nOptions:\n");
-	fprintf(stdout,"    -c, --config=config\tlocation of the configuration file\n");
-	fprintf(stdout,"    -s, --specific=file\trestore this file from an archive\n");
+	fprintf(stdout,"    -c, --config=config\t\tlocation of the configuration file\n");
+	fprintf(stdout,"    -s, --specific=file\t\trestore this file from an archive\n");
 	fprintf(stdout,"    -i, --igore_tar\t\tignore tar errors when restoring\n");
 	fprintf(stdout,"    -I, --igore_conf\t\tignore errors in the config file\n");
+	fprintf(stdout,"    -P, --patched-tar\t\ttar is patched, enables --no-recursion and more\n");
 	fprintf(stdout,"    -d, --dryrun\t\tdo a dry run, don't touch the filesystem\n");
 	fprintf(stdout,"    -V, --verbose\t\tbe more verbose\n");
-	fprintf(stdout,"    -V -V\t\tbe more verbose\n");
-	fprintf(stdout,"    -q, --quiet\t\tsuppress output from subprocesses\n");
-	fprintf(stdout,"    -q -q\tsuppress logging output\n");
-	fprintf(stdout,"    -q -q -q\t\tno logging at all, even no overview message\n");
-	fprintf(stdout,"    -h, --help\t\tthis help\n");
+	fprintf(stdout,"    -V -V\t\t\tbe even more verbose\n");
+	fprintf(stdout,"    -q, --quiet\t\t\tsuppress output from subprocesses\n");
+	fprintf(stdout,"    -q -q\t\t\tsuppress logging output\n");
+	fprintf(stdout,"    -q -q -q\t\t\tno logging at all, even no overview message\n");
+	fprintf(stdout,"    -h, --help\t\t\tthis help\n");
 	fprintf(stdout,"    -v, --version\t\tshow version\n");
-	fprintf(stdout,"    -D, --debug\t\tshow debugging information\n");
+	fprintf(stdout,"    -D, --debug\t\t\tshow debugging information\n");
 #else
 	fprintf(stdout,"\nOptions:\n");
 	fprintf(stdout,"    -c config\tlocation of the configuration file\n");
 	fprintf(stdout,"    -s file\trestore this file from an archive\n");
 	fprintf(stdout,"    -i\t\tignore tar errors when restoring\n");
 	fprintf(stdout,"    -I\t\tignore errors in the config file\n");
+	fprintf(stdout,"    -P\t\ttar is patched, enables --no-recursion and more\n");
 	fprintf(stdout,"    -d\t\tdo a dry run, don't touch the filesystem\n");
 	fprintf(stdout,"    -V\t\tbe more verbose\n");
 	fprintf(stdout,"    -V -V\tbe even more verbose\n");
@@ -1426,7 +1398,7 @@ usage()
 	fprintf(stdout,"    -h\t\tthis help\n");
 	fprintf(stdout,"    -v\t\tshow version\n");
 	fprintf(stdout,"    -D\t\tshow debugging information\n");
-#endif /*LONG*/
+#endif /*HAVE_GETOPT_H*/
 	fprintf(stdout,"\n");
 	fprintf(stdout,"Report bugs to <hdup-user@miek.nl>.\n");
 	fprintf(stdout,"Or at http://www.miek.nl/cgi-bin/bugzilla/index.cgi\n");
