@@ -24,7 +24,7 @@
 #include"prototype.h"
 
 void 
-backup_remote(phost_t host[], int which, struct argument_t *arg) 
+backup_remote(phost_t host[], int which) 
 {
 	/* receive a header from stdin, after that comes the tarfile.
 	 * From the header we can deduce what to do with the archive.
@@ -39,7 +39,7 @@ backup_remote(phost_t host[], int which, struct argument_t *arg)
 	char *tar = NULL;
 	char *split = NULL;
 	unsigned int t = 0;
-	unsigned long int b = 0;
+	unsigned long long int bytes;
 	
 	/* make the dirs */
         hdup_setup(host, which);
@@ -111,9 +111,11 @@ backup_remote(phost_t host[], int which, struct argument_t *arg)
 			out = tar;
 
 		/* our output command has been specified */
+		/* dont lock for restore
 		hdup_lock(host[which], LOCK);
+		*/
 
-		if ( stream2pipe(stdin, out, NULL) != 0 ) {
+		if (stream2pipe(stdin, out, NULL, &bytes) != 0) {
 			LOG("%s", "Could not remotely restore the archive");
 			hdup_cleanup(-1,host[which]);
 		}
@@ -140,18 +142,18 @@ backup_remote(phost_t host[], int which, struct argument_t *arg)
 		/* set the lock */
 		(void)hdup_lock(host[which], LOCK);
 		if ( host[which]->chunksize == NULL ) {
-			if ( ( b = stream2file(stdin, host[which]->archivename)) != 0 ) {
+			if (stream2file(stdin, host[which]->archivename, &bytes) == 0) {
 				(void)hdup_chown (host[which]->archivename, host[which]->user,
 						  host[which]->group);
 				(void)hdup_chmod (host[which]->archivename);
-				VERBOSE("wrote %s", hdup_humansize((long long)b));
+				VERBOSE("wrote %s", hdup_humansize((long long)bytes));
 			} else {
 				LOG("%s", "Could not create archive");
 				hdup_cleanup(-1,host[which]);
 			}
 		} else {
 			/* splitting */
-			if ( stream2pipe(stdin, split, NULL) == 0 ) {
+			if (stream2pipe(stdin, split, NULL, &bytes) == 0) {
 				t = hdup_chown_dir(host[which]->dirname_date,
 						hdup_scheme(header->scheme) , host[which]->user,
 						host[which]->group);
