@@ -63,7 +63,7 @@ backup_restore(phost_t host[], int which, struct argument_t *arg )
 		date_format = DATE_DEF;
 	}
 
-	if ((how_many = find_restore(arg,host[which],restore, date_format) ) == -1) 
+	if ((how_many = find_restore(arg,host[which], restore, date_format) ) == -1) 
 		FATAL("%s", "No archives found: nothing restored");
 
 	if (how_many > 3)
@@ -72,8 +72,6 @@ backup_restore(phost_t host[], int which, struct argument_t *arg )
 	if (how_many < 2)
 		LOG("%s", "No monthly archive found. Trying to restore from what I have");
 
-	(void) hdup_lock(host[which], LOCK);
-
 	/* the find_restore function return the archive in restore[]. I must
 	 * walk that array in the reverse order: 3 2 1. If it's not null I
 	 * should extract it. */
@@ -91,20 +89,17 @@ backup_restore(phost_t host[], int which, struct argument_t *arg )
 		}
 	}
 
-
-	for ( i = how_many; i >= DAILY; i-- ) {
+	for (i = how_many; i >= 0; i--) {
 		host[which]->date = arg->date;
 		if ( restore[i] != NULL ) {
 			VERBOSE("%s %s", "Restoring from",restore[i]);
 			stat = hdup_dountar(host[which], arg, restore[i]);
-			if (stat != 0 && host[which]->remote != NULL)
+			if (stat != 0 && host[which]->remote != NULL) {
 				/* no point in trying do something next */
 				break;
+			}
 
-			if (i != MONTHLY)
-				hdup_overview(hdup_lastname(restore[i]),NULL);
-			else 
-				hdup_overview("Archives",hdup_lastname(restore[i]));
+			hdup_overview("Archives", hdup_lastname(restore[i]));
 		}
 		/* some tar has been lucky */
 		if (stat == 0) 
