@@ -17,11 +17,6 @@
  *   along with this package; if not, write to the Free Software
  *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  *   02111-1307, USA.
- *  
- *   ChangeLog:
- *   17-mar-2003: removed all the fgets and fputs replaced them with getc/putc
- *   20-mar-2003: MUTE works again
- *
  */
 
 #include"hdup.h"
@@ -40,12 +35,15 @@ hdup_dountar(phost_t host, struct argument_t * arg, char *restore)
 	char *cat = NULL;
 	char *decrypt = NULL;
 	FILE *in = NULL;
+	unsigned long long int bytes;
+	int ret;
 
 	pheader_t header = NULL;
+	ret = 0;
 
-	if ( host->remote == NULL ) {
+	if (host->remote == NULL) {
 		/* stay local */
-		if (hdup_compression(restore) == CRYPT ) {
+		if (hdup_compression(restore) == CRYPT) {
 			/* is something is encrpypted I must have 
 			 * the decryption key */
 			if (host->alg == NULL ) { 
@@ -55,10 +53,10 @@ hdup_dountar(phost_t host, struct argument_t * arg, char *restore)
 
 			decrypt = setup_decryptcmd(host);
 
-			tar = setup_untarcmd(host,arg->extractdir,arg->extractfile, hdup_compression_crypt(restore));
+			tar = setup_untarcmd(host,arg->extractdir, arg->extractfile, hdup_compression_crypt(restore));
 			out = g_strdup_printf("%s | %s", decrypt, tar);
 		} else {
-			tar = setup_untarcmd(host,arg->extractdir,arg->extractfile, hdup_compression(restore));
+			tar = setup_untarcmd(host, arg->extractdir, arg->extractfile, hdup_compression(restore));
 			out = tar;
 		}
 	} else {
@@ -88,18 +86,21 @@ hdup_dountar(phost_t host, struct argument_t * arg, char *restore)
 			return 0;
 		}
 		
-		if (!ignore_tar_err) 
-			return ( pipe2pipe(cat, out, header) );
-		else {
+		if (!ignore_tar_err) {
+			ret = pipe2pipe(cat, out, header, &bytes);
+			host->bytes = bytes;
+			return ret;
+		} else {
 			/* discard any errors */
-			pipe2pipe(cat, out, header);
+			pipe2pipe(cat, out, header, &bytes);
+			host->bytes = bytes;
 			return 0;
 		}
 
 	} else {
 		/* ok, normal people still exist */
 		in = fopen(restore, "r");
-		if ( in == NULL ) {
+		if (in == NULL ){
 			VERBOSE("%s", "Could not open the archive for reading");
 			perror(NULL);
 			return 1;
@@ -109,14 +110,18 @@ hdup_dountar(phost_t host, struct argument_t * arg, char *restore)
 				return 0;
 			}
 			if (!ignore_tar_err) {
-				return (stream2pipe(in, out, header));
+				ret = stream2pipe(in, out, header, &bytes);
+				host->bytes = bytes;
+				return ret;
 			} else {
-				stream2pipe(in, out, header);
+				stream2pipe(in, out, header, &bytes);
+				host->bytes = bytes;
 				return 0;
 			}
 	}
 	/* should not reached this */
 	VERBOSE("%s", "restore: nothing to do?!");
+	host->bytes = 0;
 	return 1;
 }
 
@@ -136,6 +141,7 @@ hdup_dotar(phost_t host, int s)
 	char *split = NULL; /* hold entire split command */
 	pheader_t header = NULL;
 	unsigned int t = 0; 
+	unsigned long long int bytes;
 
 	/* get tar ready */
 	tar = setup_tarcmd(host);
@@ -164,9 +170,10 @@ hdup_dotar(phost_t host, int s)
 					VVERBOSE("Faking %s", tar);
 					return 0;
 				}
-				if ( pipe2file(tar, out) != 0 ) 
+				if (pipe2file(tar, out, &bytes) != 0) 
 					return 1;
 				
+				host->bytes = bytes;
 				(void)hdup_chown (out, host->user, host->group);
 				(void)hdup_chmod (out);
 				return 0;
@@ -178,9 +185,10 @@ hdup_dotar(phost_t host, int s)
 					VVERBOSE("Faking %s", tar);
 					return 0;
 				}
-				if ( pipe2pipe(tar, out, header) != 0 )
+				if (pipe2pipe(tar, out, header, &bytes) != 0)
 					return 1;
 
+				host->bytes = bytes;
 				t = hdup_chown_dir(host->dirname_date, s, host->user, 
 						host->group);
 				hdup_overview("Chunks", g_strdup_printf("%d", t));
@@ -198,9 +206,11 @@ hdup_dotar(phost_t host, int s)
 			hdup_fillheader(header, host, s, NULL);
 
 			out = ssh;
-			if ( pipe2pipe(tar, out, header) != 0 )
+			if (pipe2pipe(tar, out, header, &bytes) != 0)
 				return 1;
 
+			host->bytes = bytes;
+
 			return 0;
 		}
 	} else {
@@ -218,9 +228,10 @@ hdup_dotar(phost_t host, int s)
 					VVERBOSE("Faking %s", tar);
 					return 0;
 				}
-				if (pipe2pipe(tar,out, header) != 0 ) 
+				if (pipe2pipe(tar,out, header, &bytes) != 0) 
 					return 1;
 
+				host->bytes = bytes;
 				(void)hdup_chown (host->archivename, host->user, host->group);
 				(void)hdup_chmod (host->archivename);
 				return 0;
@@ -241,9 +252,10 @@ hdup_dotar(phost_t host, int s)
 					VVERBOSE("Faking %s", out);
 					return 0;
 				}
-				if (pipe2pipe(tar, out, header) != 0 ) 
+				if (pipe2pipe(tar, out, header, &bytes) != 0 ) 
 					return 1;
 
+				host->bytes = bytes;
 				/* it could be that the whole dir is too much */
 				t = hdup_chown_dir(host->dirname_date, s, host->user,
 						host->group);
@@ -261,23 +273,26 @@ hdup_dotar(phost_t host, int s)
 				VVERBOSE("Faking %s", tar);
 				return 0;
 			}
-			if ( pipe2pipe(in,ssh, header) != 0 ) 
+			if (pipe2pipe(in,ssh, header, &bytes) != 0) 
 				return 1;
 
+			host->bytes = bytes;
 			return 0;
 		}
 	}
+	host->bytes = 0;
 	return 0;
 }
 
 int 
-pipe2file(char *tar, char *out) 
+pipe2file(char *tar, char *out, unsigned long long int *b) 
 {
 	/* write the tar to a file */
 	FILE *pin;
 	FILE *archivefile;
-	int c; int stat;
-	char buf[BUFSIZE];
+	unsigned int c; int stat;
+	char buf[BUFSIZE + 1];
+	*b = 0;
 
 	archivefile = fopen(out,"w");
 	pin = popen(tar,"r");
@@ -304,13 +319,8 @@ pipe2file(char *tar, char *out)
                         perror(NULL);
                         return 1; 
                 }
+		*b += c;
         }
-
-#if 0
-	while((c = getc(pin)) != EOF)
-		putc(c, archivefile);
-#endif
-
 	fclose(archivefile); 
 
 	stat = pclose(pin);
@@ -331,17 +341,16 @@ pipe2file(char *tar, char *out)
 }
 
 /* cat a stream to a file
- * return the number of bytes written
  * return 0 on error
  */
-unsigned long int
-stream2file(FILE *in, char *file)
+int
+stream2file(FILE *in, char *file, unsigned long long int *b)
 {
 	/* read from the stream and write to file */
 	FILE *handle;
-	int c;
-	unsigned long int b = 0;
-	char buf[BUFSIZE];
+	unsigned int c;
+	char buf[BUFSIZE + 1];
+	*b = 0;
 
 	handle = fopen(file,"w");
 
@@ -364,37 +373,40 @@ stream2file(FILE *in, char *file)
                         perror(NULL);
                         return 1;
                 }
-                b = b + c;
+                *b += c;
         }
 
 	fclose(handle);
-	return b;
+	return 0;
 }
 
 int
-stream2pipe(FILE *in, char *out, pheader_t head) 
+stream2pipe(FILE *in, char *out, pheader_t head, unsigned long long int *b) 
 {
 	/* write from the stream and to the pipe */
 	FILE *pout;
-	int c; int stat;
-	char buf[BUFSIZE];
+	unsigned int c; int stat;
+	char buf[BUFSIZE + 1];
+	*b = 0;
 
 	fflush(stdin);
 	fflush(stdout);
 	pout = popen(out,"w");
 
-	if ( pout == NULL ) { 
+	if (pout == NULL) { 
 		VERBOSE("%s","Could not open pipe for writing"); 
 		perror(NULL);
 		return 1; 
 	}
-	if ( in == NULL ) { 
+	if (in == NULL) { 
 		VERBOSE("%s", "Could not setup stream for reading"); 
 		perror(NULL);
 		return 1;
 	}
 
-	if ( head != NULL ) {
+	/* is sigalrm helpfull here */
+
+	if (head != NULL) {
 		/* print the header first */
 		hdup_putheader(head, pout);
 	}
@@ -411,6 +423,7 @@ stream2pipe(FILE *in, char *out, pheader_t head)
                         perror(NULL);
                         return 1;
                 }
+		*b += c; 
         }
 
 	stat = pclose(pout);
@@ -426,16 +439,17 @@ stream2pipe(FILE *in, char *out, pheader_t head)
 }
 
 int 
-pipe2pipe(char *tar, char *out, pheader_t head) 
+pipe2pipe(char *tar, char *out, pheader_t head, unsigned long long int *b) 
 {
 	/* write the tar to a pipe */
 	FILE *pin;
 	FILE *pout;
-	int c; int stat;
-	char buf[BUFSIZE];
+	unsigned int c; int stat;
+	char buf[BUFSIZE + 1];
 
 	pout = popen(out,"w");
 	pin = popen(tar,"r");
+	*b = 0;
 
 	VVERBOSE("Running: %s | %s", tar, out);
 
@@ -465,6 +479,7 @@ pipe2pipe(char *tar, char *out, pheader_t head)
                         perror(NULL);
                         return 1;
                 }
+		*b += c;
         }
 
 	/* out pipe */
@@ -578,11 +593,22 @@ setup_tarcmd(phost_t host)
 	/* create a string with the tar command */
 	char *tar; char *comp;
 
-	/* tar will always look like this */
-	tar = g_strdup_printf("%s %s - %s --files-from %s --listed-incremental %s --ignore-failed-read %s",
-			host->tar, TAR_NONE, SPARSE,
-			host->filelist, host->inclist,
-			host->tar_conf_opt);
+	/* tar will always look like this 
+	 * MG 28-7-2005 use --null as we delimit are list with \0's
+	 */
+	if (patched_tar == 1) {
+		tar = g_strdup_printf("%s %s - %s --no-recursion --null --files-from %s --listed-incremental %s --ignore-failed-read %s",
+				host->tar, TAR_NONE, SPARSE,
+				host->filelist, host->inclist,
+				host->tar_conf_opt);
+
+
+	} else {
+		tar = g_strdup_printf("%s %s - %s --files-from %s --null --listed-incremental %s --ignore-failed-read %s",
+				host->tar, TAR_NONE, SPARSE,
+				host->filelist, host->inclist,
+				host->tar_conf_opt);
+	}
 
 	if (strncmp(host->compression, TAR_GZIP, strlen(TAR_GZIP)) == 0) {
 		comp = g_strdup_printf(" |%s -%s",GZIP_PROG, host->complevel);
@@ -666,10 +692,10 @@ setup_untarcmd(phost_t host, char * extractdir, char *extractfile, int compressi
 				host->tar_conf_opt);
 	} else {
 		/* extract a single file */
-		tar = g_strdup_printf("%s %s - %s %s %s --ignore-failed-read --incremental %s %s",
+		tar = g_strdup_printf("%s %s - %s %s --ignore-failed-read --incremental %s %s %s",
 				host->tar, UNTAR_OPT,  
-				extractdir, extractfile, SPARSE, tar_opt,
-				host->tar_conf_opt);
+				extractdir, SPARSE, tar_opt,
+				host->tar_conf_opt, extractfile);
 	}
 	if (quiet > 0)
 		tar = g_strconcat(tar, MUTE, NULL);
