@@ -1,15 +1,17 @@
 /* $Id: prototype.h,v 1.36 2004/09/29 09:11:28 miekg Exp $ */
 /* function prototypes */
 #include <regex.h>
+
 #include "walker.h"
 
-void backup_local(phost_t host[], int , struct argument_t *);
-void backup_restore(phost_t host[], int , struct argument_t *);
-void backup_remote(phost_t hdhost[], int, struct argument_t *);
+void backup_local(phost_t host[], int which, struct argument_t *);
+void backup_restore(phost_t host[], int which, struct argument_t *);
+void backup_remote(phost_t hdhost[], int which);
 void hdup_cleanup (int , phost_t ); 
 void hdup_print(void);
 void hdup_sigpipe(int);
 void hdup_sigint(int);
+void hdup_sigalrm(int);
 void hdup_archive(phost_t , int);
 int config(char *, phost_t host[]);
 int setlist(char *, char *hostpath[], int);
@@ -49,8 +51,8 @@ int version(void);
 int readline (char *, FILE *, int, int *);
 int usage (void);
 int find_restore (struct argument_t *, phost_t , char **, int);
-int hdup_runpre (phost_t , char *scheme[], int , char *);
-int hdup_runpost (phost_t , char *scheme[], int );
+int hdup_runpre (phost_t , char *scheme[], int);
+int hdup_runpost (phost_t , char *scheme[], int);
 char * hdup_time(time_t);
 int hdup_putheader(pheader_t, FILE * );
 int hdup_fillheader(pheader_t, phost_t, int, char *);
@@ -64,12 +66,13 @@ int hdup_ok_char(char *);
 int pure_whitespace(char *);
 long long int hdup_free(char *);    
 long long int hdup_convert(char *); 
-int pipe2file(char *, char *);
-int pipe2pipe(char *, char *, pheader_t);
-unsigned long int stream2file(FILE *, char *);
-int stream2pipe(FILE *, char *, pheader_t);
+
+int pipe2file(char *, char *, unsigned long long int *);
+int pipe2pipe(char *, char *, pheader_t, unsigned long long int *);
+int stream2file(FILE *, char *, unsigned long long int *);
+int stream2pipe(FILE *, char *, pheader_t, unsigned long long int *);
+
 char * hdup_subst(char *, char *, char *);
-char * hdup_size(char *);
 char * hdup_humansize(long long);
 char * hdup_lastname(char * );         /* ala basename */
 char * day(char *, int , int );
@@ -82,7 +85,6 @@ char * setup_decryptcmd(phost_t);
 char * setup_splitcmd(phost_t);
 char * hdup_makedate(int , int , int , int );
 time_t hdup_file_time(char *);
-int isblank(int c);
 /* 
  * walker.c 
  */
